@@ -5,12 +5,10 @@ mod image_cache;
 mod positioning_tests;
 pub mod text;
 mod text_run_manager;
-mod vertex_pool;
 
 use crate::components::core::orthographic_projection;
 use crate::components::rich_text::image_cache::{GlyphCache, ImageCache};
 use crate::components::rich_text::text_run_manager::{CacheResult, TextRunManager};
-use crate::components::rich_text::vertex_pool::VertexPool;
 use crate::context::Context;
 use crate::font::FontLibrary;
 use crate::font_introspector::GlyphId;
@@ -52,7 +50,6 @@ pub struct RichTextBrush {
     images: ImageCache,
     glyphs: GlyphCache,
     text_run_manager: TextRunManager,
-    vertex_pool: VertexPool,
 }
 
 impl RichTextBrush {
@@ -269,35 +266,9 @@ impl RichTextBrush {
             vertex_buffer,
             supported_vertex_buffer,
             current_transform,
-            vertex_pool: VertexPool::new(),
         }
     }
 
-    /// Warm the text run cache with common terminal patterns
-    /// This should be called during initialization to improve cache hit rates
-    pub fn warm_cache(&mut self, _font_library: &FontLibrary) {
-        // Use the default font ID (0) which is always available
-        let font_id = 0; // FONT_ID_REGULAR
-        let font_size = 14.0; // Default terminal font size
-        let default_color = [1.0, 1.0, 1.0, 1.0]; // White text
-
-        self.text_run_manager.warm_cache(
-            font_id,
-            font_size,
-            400, // Normal weight
-            0,   // Normal style
-            5,   // Normal stretch
-            default_color,
-        );
-    }
-
-    /// Get vertex pool statistics for performance monitoring
-    pub fn vertex_pool_stats(
-        &self,
-    ) -> Option<crate::components::rich_text::vertex_pool::VertexPoolStats> {
-        self.vertex_pool.stats()
-    }
-
     #[inline]
     pub fn prepare(
         &mut self,
@@ -350,10 +321,6 @@ impl RichTextBrush {
         self.vertices.clear();
         self.images.process_atlases(context);
         self.comp.finish(&mut self.vertices);
-
-        // Performance monitoring
-        self.text_run_manager.maintenance();
-        self.vertex_pool.maybe_log_stats();
     }
 
     #[inline]
@@ -363,34 +330,46 @@ impl RichTextBrush {
         render_data: &crate::layout::BuilderLine,
         graphics: &mut Graphics,
     ) -> Option<SugarDimensions> {
-        // Every dimension request will lead to clear text run cache if needed
-        if self.text_run_manager.needs_cleanup() {
-            self.text_run_manager.maintenance();
-        }
         self.comp.begin();
 
         let lines = vec![render_data.clone()];
         self.draw_layout(0, &lines, &None, None, font_library, None, graphics)
     }
 
-    fn extract_font_metrics(
+    /// Extract font metrics using per-font calculation.
+    /// Each font calculates its own metrics using consistent approach.
+    #[inline]
+    fn extract_normalized_metrics(
+        &self,
         lines: &[crate::layout::BuilderLine],
+        font_library: &FontLibrary,
     ) -> Option<(f32, f32, f32, usize, f32)> {
-        // Extract the first run from a line that has at least one run
-        lines
+        // Get the first run to determine font_id and size
+        let first_run = lines
             .iter()
             .filter(|line| !line.render_data.runs.is_empty())
             .map(|line| &line.render_data.runs[0])
-            .next()
-            .map(|run| {
-                (
-                    run.ascent.round(),
-                    run.descent.round(),
-                    (run.leading).round() * 2.0,
-                    run.span.font_id,
-                    run.size,
-                )
-            })
+            .next()?;
+
+        let font_id = 0; // FONT_ID_REGULAR
+        let font_size = first_run.size;
+
+        // Get metrics from the specific font using consistent calculation
+        let mut font_library_data = font_library.inner.write();
+        if let Some((ascent, descent, leading)) =
+            font_library_data.get_font_metrics(&font_id, font_size)
+        {
+            Some((ascent, descent, leading, font_id, font_size))
+        } else {
+            // Fallback to run metrics if font metrics calculation fails
+            Some((
+                first_run.ascent,
+                first_run.descent,
+                first_run.leading,
+                font_id,
+                font_size,
+            ))
+        }
     }
 
     #[inline]
@@ -409,13 +388,6 @@ impl RichTextBrush {
             return None;
         }
 
-        // let start = std::time::Instant::now();
-        let comp = &mut self.comp;
-        let caches = (&mut self.images, &mut self.glyphs);
-        let (image_cache, glyphs_cache) = caches;
-        let font_coords: &[i16] = &[0, 0, 0, 0];
-        let depth = 0.0;
-
         // Determine if we're calculating dimensions only or drawing layout
         let is_dimensions_only = pos.is_none() || rte_layout.is_none();
 
@@ -426,6 +398,17 @@ impl RichTextBrush {
             lines.as_slice()
         };
 
+        // Extract font metrics before borrowing self.comp
+        let font_metrics =
+            self.extract_normalized_metrics(lines_to_process, font_library);
+
+        // let start = std::time::Instant::now();
+        let comp = &mut self.comp;
+        let caches = (&mut self.images, &mut self.glyphs);
+        let (image_cache, glyphs_cache) = caches;
+        let font_coords: &[i16] = &[0, 0, 0, 0];
+        let depth = 0.0;
+
         // Get initial position
         let (x, y) = pos.unwrap_or((0.0, 0.0));
 
@@ -434,8 +417,6 @@ impl RichTextBrush {
         let mut last_rendered_graphic = HashSet::new();
         let mut line_y = y;
         let mut dimensions = SugarDimensions::default();
-
-        let font_metrics = Self::extract_font_metrics(lines_to_process);
         if let Some((
             ascent,
             descent,
@@ -515,9 +496,6 @@ impl RichTextBrush {
                             &run_text,
                             font,
                             run.size,
-                            400, // font_weight - would need to be extracted from font
-                            0,   // font_style - would need to be extracted from font
-                            5,   // font_stretch - would need to be extracted from font
                             Some(run.span.color),
                         )
                     } else {
@@ -673,9 +651,6 @@ impl RichTextBrush {
                                     &run_text,
                                     font,
                                     run.size,
-                                    400, // font_weight
-                                    0,   // font_style
-                                    5,   // font_stretch
                                     shaped_glyphs,
                                     false, // has_emoji - would need to be detected
                                     None, // shaping_features - would need actual shaping data
@@ -773,6 +748,10 @@ impl RichTextBrush {
             }
         }
 
+        // let screen_render_duration = start.elapsed();
+        // if self.renderer.enable_performance_logging {
+        // println!("[PERF] draw_layout() total: {:?}", screen_render_duration);
+
         // Return dimensions if in dimensions mode
         if is_dimensions_only {
             if dimensions.height > 0.0 && dimensions.width > 0.0 {
@@ -805,6 +784,7 @@ impl RichTextBrush {
         ctx: &mut Context,
         rpass: &mut wgpu::RenderPass<'pass>,
     ) {
+        // let start = std::time::Instant::now();
         // There's nothing to render
         if self.vertices.is_empty() {
             return;
@@ -867,9 +847,10 @@ impl RichTextBrush {
         rpass.set_bind_group(1, &self.layout_bind_group, &[]);
         rpass.set_vertex_buffer(0, self.vertex_buffer.slice(..));
 
-        // Use draw instead of draw_indexed
         let vertex_count = self.vertices.len() as u32;
         rpass.draw(0..vertex_count, 0..1);
+        // let duration = start.elapsed();
+        // println!("Time elapsed in rich_text::render is: {:?}", duration);
     }
 }
 
